@@ -1,9 +1,9 @@
 /*****
 *
-* Copyright (C) 2001, 2003 Yoann Vandoorselaere <yoann@prelude-ids.org>
-* All Rights Reserved
+* Copyright (C) 2001, 2002, 2003, 2004, 2005 PreludeIDS Technologies. All Rights Reserved.
+* Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
-* This file is part of the Prelude program.
+* This file is part of the Prelude library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -24,41 +24,35 @@
 #ifndef _LIBPRELUDE_PRELUDE_IO_H
 #define _LIBPRELUDE_PRELUDE_IO_H
 
-#include <inttypes.h>
-
-#ifdef SWIG
-
-%module prelude_io
-
-typedef void FILE;
-typedef int size_t;
-typedef int ssize_t;
-typedef unsigned short int uint16_t;
-
-%{
-	#include "prelude-io.h"
-%}
-
+#ifdef __cplusplus
+  extern "C" {
 #endif
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+          
+#include <stdio.h>
+#include <unistd.h>
+#include "prelude-inttypes.h"
+
+
 typedef struct prelude_io prelude_io_t;
 
 /*
  * Object creation / destruction functions.
  */
-prelude_io_t *prelude_io_new(void);
+int prelude_io_new(prelude_io_t **ret);
 
 void prelude_io_destroy(prelude_io_t *pio);
 
 void prelude_io_set_file_io(prelude_io_t *pio, FILE *fd);
 
-void prelude_io_set_ssl_io(prelude_io_t *pio, void *ssl);
-
-void prelude_io_set_socket_io(prelude_io_t *pio, int fd);
+void prelude_io_set_tls_io(prelude_io_t *pio, void *tls);
 
 void prelude_io_set_sys_io(prelude_io_t *pio, int fd);
 
-
+int prelude_io_set_buffer_io(prelude_io_t *pio);
 
 
 /*
@@ -75,7 +69,7 @@ ssize_t prelude_io_read_delimited(prelude_io_t *pio, unsigned char **buf);
 
 ssize_t prelude_io_write(prelude_io_t *pio, const void *buf, size_t count);
 
-int prelude_io_write_delimited(prelude_io_t *pio, const void *buf, uint16_t count);
+ssize_t prelude_io_write_delimited(prelude_io_t *pio, const void *buf, uint16_t count);
 
 
 ssize_t prelude_io_forward(prelude_io_t *dst, prelude_io_t *src, size_t count);
@@ -86,4 +80,9 @@ void *prelude_io_get_fdptr(prelude_io_t *pio);
 
 ssize_t prelude_io_pending(prelude_io_t *pio);
 
+          
+#ifdef __cplusplus
+  }
+#endif
+        
 #endif /* _LIBPRELUDE_PRELUDE_IO_H */
