@@ -1,9 +1,9 @@
 /*****
 *
-* Copyright (C) 2001, 2002, 2003 Yoann Vandoorselaere <yoann@prelude-ids.org>
-* All Rights Reserved
+* Copyright (C) 2001-2004 PreludeIDS Technologies. All Rights Reserved.
+* Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
-* This file is part of the Prelude program.
+* This file is part of the Prelude library.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -21,31 +21,51 @@
 *
 *****/
 
+#include "config.h"
+
 #include <stdio.h>
+#include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
 #include <sys/poll.h>
 #include <sys/ioctl.h>
-#include <inttypes.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <termios.h>
+#include <gnutls/gnutls.h>
 
-#include "config.h"
+#include "prelude-inttypes.h"
 
-#ifdef HAVE_SSL
- #include <openssl/ssl.h>
- #include <openssl/err.h>
+#ifdef HAVE_SYS_FILIO_H
+# include <sys/filio.h>
 #endif
 
+
 #include "prelude-log.h"
 #include "prelude-io.h"
+#include "common.h"
+
+
+#define PRELUDE_ERROR_SOURCE_DEFAULT PRELUDE_ERROR_SOURCE_IO
+#include "prelude-error.h"
+
+
+#define CHUNK_SIZE 1024
+
+#ifndef MIN
+# define MIN(x, y) ((x) < (y) ? (x) : (y))
+#endif
 
 
 struct prelude_io {
+
         int fd;
         void *fd_ptr;
         
+        size_t size;
+        size_t rindex;
+        
         int (*close)(prelude_io_t *pio);
         ssize_t (*read)(prelude_io_t *pio, void *buf, size_t count);
         ssize_t (*write)(prelude_io_t *pio, const void *buf, size_t count);
@@ -54,60 +74,66 @@ struct prelude_io {
 
 
 
-
 /*
- * Check if the tcp connection has been closed by peer
- * i.e if peer has sent a FIN tcp segment.
- *
- * It is important to call this function before writing on
- * a tcp socket, otherwise the write will succeed despite
- * the remote socket has been closed and next write will lead
- * to a broken pipe
+ * Buffer IO functions.
  */
-static int is_tcp_connection_still_established(int fd)
+static ssize_t buffer_read(prelude_io_t *pio, void *buf, size_t count)
 {
-        int pending;
-	struct pollfd pfd;
+        if ( pio->size - pio->rindex == 0 )
+                return 0;
 
-	pfd.fd = fd;
-	pfd.events = POLLIN;
+        count = MIN(count, pio->size - pio->rindex);
 
-	if ( poll(&pfd, 1, 0) < 0 ) {
-		log(LOG_ERR, "poll on tcp socket failed.\n");
-		return -1;
-	}
-
-	if ( pfd.revents & POLLERR ) {
-		log(LOG_ERR, "error polling tcp socket.\n");
-		return -1;
-	}
-
-	if ( pfd.revents & POLLHUP ) {
-		log(LOG_ERR, "connection hang up.\n");
-		return -1;
-	}
-
-	if ( ! (pfd.revents & POLLIN) )
-		return 0;
-
-	/*
-	 * Get the number of bytes to read
-	 */
-	if ( ioctl(fd, FIONREAD, &pending) < 0 ) {
-		log(LOG_ERR, "ioctl FIONREAD failed on tcp socket.\n");
-		return -1;
-	}
-
-	if ( ! pending ) {
-		log(LOG_ERR, "connection has been closed by peer.\n");
-		return -1;
-	}
-
-	return 0;
+        memcpy(buf, (unsigned char *) pio->fd_ptr + pio->rindex, count);
+        pio->rindex += count;
+        
+        return count;
 }
 
 
 
+static ssize_t buffer_write(prelude_io_t *pio, const void *buf, size_t count)
+{
+        unsigned char *new;
+        
+        if ( pio->size + count <= pio->size )
+                return -1;
+        
+        new = _prelude_realloc(pio->fd_ptr, pio->size + count);
+        if ( ! new )
+                return prelude_error_from_errno(errno);
+        
+        memcpy(new + pio->size, buf, count);
+
+        pio->fd_ptr = new;
+        pio->size += count;
+        
+        return count;
+}
+
+
+
+static int buffer_close(prelude_io_t *pio)
+{
+        if ( pio->fd_ptr ) {
+                free(pio->fd_ptr);
+                pio->fd_ptr = NULL;
+                pio->size = pio->rindex = 0;
+        }
+
+        return 0;
+}
+
+
+
+static ssize_t buffer_pending(prelude_io_t *pio)
+{
+        return pio->size - pio->rindex;
+}
+
+
+
+
 /*
  * System IO functions.
  */
@@ -116,8 +142,14 @@ static ssize_t sys_read(prelude_io_t *pio, void *buf, size_t count)
         ssize_t ret;
 
         do {
-                ret = read(pio->fd, buf, count);        
-        } while ( ret < 0 && (errno == EINTR || errno == EAGAIN) );
+                ret = read(pio->fd, buf, count);
+        } while ( ret < 0 && errno == EINTR );
+        
+        if ( ret == 0 )
+                return prelude_error(PRELUDE_ERROR_EOF);
+        
+        if ( ret < 0 )
+                return prelude_error_from_errno(errno);
         
         return ret;
 }
@@ -130,8 +162,11 @@ static ssize_t sys_write(prelude_io_t *pio, const void *buf, size_t count)
         
         do {
                 ret = write(pio->fd, buf, count);
-        } while ( ret < 0 && (errno == EINTR || errno == EAGAIN) );
+        } while ( ret < 0 && errno == EINTR );
 
+        if ( ret < 0 )
+                return prelude_error_from_errno(errno);
+        
         return ret;
 }
 
@@ -143,18 +178,22 @@ static int sys_close(prelude_io_t *pio)
 
         do {
                 ret = close(pio->fd);
-        } while ( ret < 0 && (errno == EINTR || errno == EAGAIN) );
+        } while ( ret < 0 && errno == EINTR );
 
-        return ret;
+        return (ret >= 0) ? ret : prelude_error_from_errno(errno);
 }
 
 
 
 static ssize_t sys_pending(prelude_io_t *pio) 
 {
-        return -1;
-}
+        ssize_t ret;
+        
+        if ( ioctl(pio->fd, FIONREAD, &ret) < 0 )
+                return prelude_error_from_errno(errno);
 
+        return ret;
+}
 
 
 
@@ -164,12 +203,21 @@ static ssize_t sys_pending(prelude_io_t *pio)
  */
 static ssize_t file_read(prelude_io_t *pio, void *buf, size_t count) 
 {
+        FILE *fd; 
         size_t ret;
-        
-        ret = fread(buf, count, 1, pio->fd_ptr);
-        if ( ret <= 0 )
-                return ret;
 
+        /*
+         * ferror / clearerror can be macro that might dereference fd_ptr.
+         */
+        fd = pio->fd_ptr;
+        
+        ret = fread(buf, count, 1, fd);        
+        if ( ret <= 0 ) {        
+                ret = ferror(fd) ? prelude_error_from_errno(errno) : prelude_error(PRELUDE_ERROR_EOF);
+                clearerr(fd);
+                return ret;
+        }
+        
         /*
          * fread return the number of *item* read.
          */
@@ -208,145 +256,117 @@ static ssize_t file_pending(prelude_io_t *pio)
 
 
 
-
 /*
- * Socket IO functions.
+ * TLS IO functions
  */
-static ssize_t socket_pending(prelude_io_t *pio)
+static int tls_check_error(prelude_io_t *pio, int ret)
 {
-        int ret;
+        const char *alert;
         
-        if ( ioctl(pio->fd, FIONREAD, &ret) < 0 ) {
-                log(LOG_ERR, "ioctl FIONREAD failed on tcp socket.\n");
-                return -1;
-        }
-
-        return ret;
-}
-
-
-
-static ssize_t socket_write(prelude_io_t *pio, const void *buf, size_t count)
-{
-	ssize_t ret;
-
-	if ( is_tcp_connection_still_established(pio->fd) < 0 )
-		return -1;
-
-        do {
-                ret = write(pio->fd, buf, count);
-        } while ( ret < 0 && (errno == EINTR || errno == EAGAIN) );
-
-	return ret;
-}
-
-
-
-
-/*
- * SSL IO functions
- */
-
-#ifdef HAVE_SSL
-
-static int handle_ssl_error(SSL *ssl, int ret, int errnum) 
-{
-        int ssl_error;
-
-        ssl_error = SSL_get_error(ssl, ret);
-        
-        switch (ssl_error) {
-                
-        case SSL_ERROR_WANT_READ:
-        case SSL_ERROR_WANT_WRITE:
-                return 1;
-
-        case SSL_ERROR_ZERO_RETURN:
+        if ( ret == GNUTLS_E_INTERRUPTED )
                 return 0;
 
-        case SSL_ERROR_SYSCALL:
-                if ( ret == 0 )
-                        /*
-                         * an EOF was observed that violates the protocol
-                         */
-                        return 0;
-                
-                if ( ret == -1 && (errnum == EAGAIN || errnum == EINTR) )
-                        /*
-                         * we got interrupted, let's try again.
-                         */
-                        return 1;
-
-                return -1;
-                
-        default:
-                log(LOG_ERR, "SSL error : %s.\n", ERR_reason_error_string(ERR_get_error()));
-                return -1;
+        else if ( ret == GNUTLS_E_AGAIN )
+                return prelude_error(PRELUDE_ERROR_EAGAIN);
+        
+        else if ( ret == GNUTLS_E_WARNING_ALERT_RECEIVED ) {
+                alert = gnutls_alert_get_name(gnutls_alert_get(pio->fd_ptr));
+                prelude_log(PRELUDE_LOG_WARN, "TLS: received warning alert: %s.\n", alert);
+                return 0;
         }
+        
+        else if ( ret == GNUTLS_E_FATAL_ALERT_RECEIVED ) {
+                alert = gnutls_alert_get_name(gnutls_alert_get(pio->fd_ptr));
+                prelude_log(PRELUDE_LOG_ERR, "TLS: received fatal alert: %s.\n", alert);
+        }
+
+        else {
+                if ( ret == GNUTLS_E_UNEXPECTED_PACKET_LENGTH ) {
+                        prelude_log(PRELUDE_LOG_DEBUG, "TLS: %s.\n", gnutls_strerror(ret));
+#ifdef ECONNRESET
+                        return prelude_error_from_errno(ECONNRESET);
+#else
+                        return prelude_error(PRELUDE_ERROR_EOF);
+#endif
+                }
+                
+                else if ( ret == GNUTLS_E_INVALID_SESSION ) {
+                        prelude_log(PRELUDE_LOG_DEBUG, "TLS: %s.\n", gnutls_strerror(ret));
+                        return prelude_error(PRELUDE_ERROR_EOF);
+                }
+                
+                prelude_log(PRELUDE_LOG_ERR, "TLS: %s.\n", gnutls_strerror(ret));
+        }
+        
+        return prelude_error(PRELUDE_ERROR_TLS);
 }
 
 
 
-static ssize_t ssl_read(prelude_io_t *pio, void *buf, size_t count) 
+static ssize_t tls_read(prelude_io_t *pio, void *buf, size_t count) 
 {
-        int ret;
+        ssize_t ret;
         
         do {
-                ret = SSL_read(pio->fd_ptr, buf, count);
-                
-        } while ( ret <= 0 && (ret = handle_ssl_error(pio->fd_ptr, ret, errno)) == 1);
+                ret = gnutls_record_recv(pio->fd_ptr, buf, count);
+
+        } while ( ret < 0 && (ret = tls_check_error(pio, ret)) == 0 );
+
+        if ( ret == 0 )
+                return prelude_error(PRELUDE_ERROR_EOF);
         
         return ret;
 }
 
 
 
-static ssize_t ssl_write(prelude_io_t *pio, const void *buf, size_t count) 
+static ssize_t tls_write(prelude_io_t *pio, const void *buf, size_t count) 
+{
+        ssize_t ret;
+
+        do {        
+                ret = gnutls_record_send(pio->fd_ptr, buf, count);
+                                
+        } while ( ret < 0 && (ret = tls_check_error(pio, ret)) == 0 );
+        
+        return ret;
+}
+
+
+
+static int tls_close(prelude_io_t *pio) 
 {
         int ret;
 
-	if ( is_tcp_connection_still_established(pio->fd) < 0 )
-		return -1;
+        do {
+                ret = gnutls_bye(pio->fd_ptr, GNUTLS_SHUT_RDWR);
+        } while ( ret < 0 && (ret = tls_check_error(pio, ret)) == 0 );
 
-        do {        
-                ret = SSL_write(pio->fd_ptr, buf, count);
-
-        } while ( ret < 0 && (ret = handle_ssl_error(pio->fd_ptr, ret, errno)) == 1);
+        if ( ret < 0 && prelude_error_get_code(ret) == PRELUDE_ERROR_EAGAIN )
+                return ret;
         
-        return ret;
-}
-
-
-
-static int ssl_close(prelude_io_t *pio) 
-{        
-        SSL_shutdown(pio->fd_ptr);
-        SSL_free(pio->fd_ptr);
+        gnutls_deinit(pio->fd_ptr);
         
         return sys_close(pio);
 }
 
 
 
-static ssize_t ssl_pending(prelude_io_t *pio)
+static ssize_t tls_pending(prelude_io_t *pio)
 {
         ssize_t ret;
         
-        ret = SSL_pending(pio->fd_ptr);
+        ret = gnutls_record_check_pending(pio->fd_ptr);
         if ( ret > 0 )
                 return ret;
         
-        ret = socket_pending(pio);
+        ret = sys_pending(pio);
         if ( ret > 0 )
                 return ret;
         
         return 0;
 }
 
-#endif
-
-
-
 
 
 /*
@@ -489,8 +509,6 @@ ssize_t prelude_io_read_wait(prelude_io_t *pio, void *buf, size_t count)
 
 
                        
-
-
 /**
  * prelude_io_read_delimited:
  * @pio: Pointer to a #prelude_io_t object.
@@ -514,30 +532,29 @@ ssize_t prelude_io_read_wait(prelude_io_t *pio, void *buf, size_t count)
  */
 ssize_t prelude_io_read_delimited(prelude_io_t *pio, unsigned char **buf) 
 {
-        int ret;
+        ssize_t ret;
         size_t count;
         uint16_t msglen;
         
         ret = prelude_io_read_wait(pio, &msglen, sizeof(msglen));
         if ( ret <= 0 ) {
-                log(LOG_ERR, "couldn't read len message of %d bytes.\n", sizeof(msglen));
+                prelude_log(PRELUDE_LOG_ERR, "couldn't read len message of %d bytes.\n", sizeof(msglen));
                 return ret;
         }
 
         count = ntohs(msglen);
 
         *buf = malloc(count);
-        if ( ! *buf ) {
-                log(LOG_ERR, "couldn't allocate %d bytes.\n", count);
-                return -1;
-        }       
+        if ( ! *buf )
+                return prelude_error_from_errno(errno);
         
         ret = prelude_io_read_wait(pio, *buf, count);
-        if ( ret <= 0 ) {
-                log(LOG_ERR, "couldn't read %d bytes.\n", count);
-                return ret;
-        }
+        if ( ret < 0 )
+                return prelude_error_from_errno(errno);
 
+        if ( ret == 0 )
+                return prelude_error(PRELUDE_ERROR_EOF);
+        
         return count;
 }
 
@@ -593,7 +610,7 @@ ssize_t prelude_io_write(prelude_io_t *pio, const void *buf, size_t count)
  * indicates nothing was written). On error, -1 is returned, and errno
  * is set appropriately.
  */
-int prelude_io_write_delimited(prelude_io_t *pio, const void *buf, uint16_t count) 
+ssize_t prelude_io_write_delimited(prelude_io_t *pio, const void *buf, uint16_t count) 
 {
         int ret;
         uint16_t nlen;
@@ -635,20 +652,19 @@ int prelude_io_close(prelude_io_t *pio)
 
 /**
  * prelude_io_new:
+ * @ret: Pointer where to store the created #prelude_io_t object.
  *
  * Create a new prelude IO object.
  *
- * Returns: a #prelude_io_t object or NULL on error.
+ * Returns: 0 on success, or a negative value if an error occur.
  */
-prelude_io_t *prelude_io_new(void) 
+int prelude_io_new(prelude_io_t **ret) 
 {
-        prelude_io_t *new;
-
-        new = malloc(sizeof(*new));
-        if ( ! new )
-                return NULL;
+        *ret = malloc(sizeof(**ret));
+        if ( ! *ret )
+                return prelude_error_from_errno(errno);
         
-        return new;
+        return 0;
 }
 
 
@@ -673,27 +689,32 @@ void prelude_io_set_file_io(prelude_io_t *pio, FILE *fdptr)
 
 
 
-#ifdef HAVE_SSL
 
 /**
- * prelude_io_set_ssl_io:
+ * prelude_io_set_tls_io:
  * @pio: A pointer on the #prelude_io_t object.
- * @ssl: Pointer on the SSL structure holding the TLS/SSL connection data.
+ * @tls: Pointer on the TLS structure holding the TLS connection data.
  *
- * Setup the @pio object to work with SSL based I/O function.
- * The @pio object is then associated with @ssl.
+ * Setup the @pio object to work with TLS based I/O function.
+ * The @pio object is then associated with @tls.
  */
-void prelude_io_set_ssl_io(prelude_io_t *pio, void *ssl) 
+void prelude_io_set_tls_io(prelude_io_t *pio, void *tls) 
 {
-        pio->fd = SSL_get_fd(ssl);
-        pio->fd_ptr = ssl;
-        pio->read = ssl_read;
-        pio->write = ssl_write;
-        pio->close = ssl_close;
-        pio->pending = ssl_pending;
+        union {
+                void *ptr;
+                int fd;
+        } data;
+        
+        data.ptr = gnutls_transport_get_ptr(tls);
+        pio->fd = data.fd;
+        
+        pio->fd_ptr = tls;
+        pio->read = tls_read;
+        pio->write = tls_write;
+        pio->close = tls_close;
+        pio->pending = tls_pending;
 }
 
-#endif
 
 
 
@@ -717,22 +738,17 @@ void prelude_io_set_sys_io(prelude_io_t *pio, int fd)
 
 
 
-/**
- * prelude_io_set_socket_io:
- * @pio: A pointer on the #prelude_io_t object.
- * @fd: A socket descriptor.
- *
- * Setup the @pio object to work with system based I/O function.
- * The @pio object is then associated with @fd.
- */
-void prelude_io_set_socket_io(prelude_io_t *pio, int fd) 
+int prelude_io_set_buffer_io(prelude_io_t *pio)
 {
-        pio->fd = fd;
         pio->fd_ptr = NULL;
-        pio->read = sys_read;
-        pio->write = socket_write;
-        pio->close = sys_close;
-        pio->pending = socket_pending;
+        pio->size = pio->rindex = 0;
+        
+        pio->read = buffer_read;
+        pio->write = buffer_write;
+        pio->close = buffer_close;
+        pio->pending = buffer_pending;
+        
+        return 0;
 }
 
 
@@ -749,11 +765,12 @@ int prelude_io_get_fd(prelude_io_t *pio)
 }
 
 
+
 /**
  * prelude_io_get_fdptr:
  * @pio: A pointer on a #prelude_io_t object.
  *
- * Returns: Pointer associated with this object (file, ssl, or NULL).
+ * Returns: Pointer associated with this object (file, tls, buffer, or NULL).
  */
 void *prelude_io_get_fdptr(prelude_io_t *pio) 
 {
@@ -781,10 +798,10 @@ void prelude_io_destroy(prelude_io_t *pio)
  * @pio: Pointer to a #prelude_io_t object.
  *
  * prelude_io_pending return the number of bytes waiting to
- * be read on an SSL or socket fd.
+ * be read on an TLS or socket fd.
  *
  * Returns: Number of byte waiting to be read on @pio, or -1
- * if @pio is not of type SSL or socket. 
+ * if @pio is not of type TLS or socket. 
  */
 ssize_t prelude_io_pending(prelude_io_t *pio)
 {
